/* 
 * Copyright 2008 TKK/ComNet
 * Released under GPLv3. See LICENSE.txt for details. 
 */
package routing;

import java.util.HashMap;

import core.Connection;
import core.Message;
import core.Settings;
import core.SimClock;
import eg.ForemostJourneyHash;

/**
 * Epidemic message router with drop-oldest buffer and only single transferring
 * connections at a time.
 */
public class ForemostRouter extends ActiveRouter {
	
	static ForemostJourneyHash nodesHash = new ForemostJourneyHash();
	static HashMap<String, Integer> messagesNodesHash;
	
	
	
	/**
	 * Constructor. Creates a new message router based on the settings in
	 * the given Settings object.
	 * @param s The settings object
	 */
	public ForemostRouter(Settings s) {
		super(s);
		messagesNodesHash = nodesHash.getMessages();
	}
	
	/**
	 * Copyconstructor.
	 * @param r The router prototype where setting values are copied from
	 */
	protected ForemostRouter(ForemostRouter r) {
		super(r);
		messagesNodesHash = nodesHash.getMessages();
	}
			
	@Override
	public void update() {
		super.update();
		if (isTransferring() || !canStartTransfer()) {
			return; // transferring, don't try other connections yet
		}
		
        exchangeByHashTable();
	}
	
	
	protected void exchangeByHashTable() {
		for (Message m : this.getMessageCollection()) {
			Integer nodeTo = messagesNodesHash.get(m.getId()+
					"N"+this.getHost().getAddress()+
					"T"+SimClock.getIntTime());
			if (nodeTo == null) continue;
			for (Connection conn : this.getConnections()) {
				if (conn.getOtherNode(this.getHost()).getAddress() == nodeTo) {
					int retVal = startTransfer(m, conn); 
					if (retVal == RCV_OK) {
						return;	// accepted a message, don't try others
					}
				}
			}
		}
	}

	@Override
	public ForemostRouter replicate() {
		return new ForemostRouter(this);
	}

}